/*
 * Copyright (c) 1995 The Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 * 	This product includes software developed by the Network Research
 * 	Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static char rcsid[] =
    "@(#) $Header: /usr/src/mash/repository/mash/mash-1/codec/audio/transcoder-adpcm.cc,v 1.3 1997/07/31 21:33:19 elan Exp $ (LBL)";
#endif

#include "inet.h"
#include "transcoder.h"
#include "transcoder-pcm.h"
extern "C" {
#include "adpcm.h"
}

class ADPCMTranscoder : public PCMTranscoder {
public:
	ADPCMTranscoder();
	virtual void recv_data(pktbuf* pb);
protected:
	adpcm_state rstate_;

};

static class ADPCMTranscoderClass : public TclClass {
public:
	ADPCMTranscoderClass() : TclClass("Transcoder/ADPCM") {}
	TclObject* create(int, const char*const*) {
		return (new ADPCMTranscoder);
	}
} adpcm_transcoder_class;

ADPCMTranscoder::ADPCMTranscoder()
{
	memset(&rstate_, 0, sizeof(rstate_));
}

void ADPCMTranscoder::recv_data(pktbuf* pb)
{
	rtphdr* rh = (rtphdr*)pb->dp;
	int cc = pb->len - sizeof(rtphdr);
	const u_char* bp = (const u_char*)(rh + 1);

	u_char wrkbuf[AUDIO_FRAMESIZE*8];

	if ((cc = (cc - sizeof(struct adpcm_state)) << 1) <= 0 ||
	    cc > sizeof(wrkbuf))
		return;

	double now;
	if (!txonly_) {
		if (bps_ == 0) {
			pb->release();
			return;
		}
		now = gettimeofday();
	}

	if (fc_ <= now || txonly_) {
		/* If we have fallen behind (>200ms), re-sync. */
		if (now - fc_ > 200000.) 	
			fc_ = now;

		NTOHS(((struct adpcm_state*)bp)->valprev);
		adpcm_decoder(bp + sizeof(struct adpcm_state),
			      wrkbuf, cc, (struct adpcm_state*)bp);

		AudioFrame af(ntohl(rh->rh_ts), wrkbuf, cc);
		int nb = encoder_->nb();
		encoder_->recv(&af);
		obytes_ = encoder_->nb();
		nb = obytes_ - nb;
		double bits = 8 * nb;
		lastfc_ = fc_;
		fc_ += 1e6 * bits / bps_;
		ofrms_++;
	}
	pb->release();
}


