#ifndef mc_manager_h
#define mc_manager_h

#include "tclcl.h"
#include "tcpchan.h"

#include "srm/srm.h"
#include "srm/appmgr-srm.h"
#include "srm/session-srm.h"

#include <string.h>   // for strcpy()

class MC_Session;
class MC_Request;
class MC_Reply;

/*
 * The MC_Manager is MASHCast's subclass from SRM_AppMgr. It
 * defines a number of virtual methods called by the SRM objects. 
 */
class CacheBuf {
public:
	CacheBuf* next_;
	CacheBuf(const char* key) {
		len_ = -1; /* unknown length */
		data_ = 0;
		off_ = 0;
		request_ = 0;
		key_ = new char[strlen(key) + 1];	    
		strcpy(key_, key);
	}
	~CacheBuf() {
		delete[] data_;
		delete[] key_;
	}
	void change_key(const char *newkey) {
		delete[] key_;
		key_ = new char[strlen(newkey) + 1];
		strcpy(key_, newkey);
	}
	inline void set(const char* data, int len) {
		delete[] data_;
		len_ = len;
		data_ = new char[len];
		memcpy(data_, data, len);
	}
	inline void copyin(int off, const char* p, int len) {
		memcpy(&data_[off], p, len);
	}
	inline void length(int len) {
		delete [] data_;
		if (len != -1) {
			/* unknown length */
			data_ = new char[len];
		}
		len_ = len;
	}
	inline void request(MC_Request *request) {
		request_ = request;
	}
	inline void start() { off_ = 0; }
	inline void adjust(int cc) { off_ += cc; }
	inline int offset() const { return (off_); }
	inline int length() const { return (len_); }
	inline char* data() const { return (data_); }
	inline char* key() const { return (key_); }
	inline MC_Request* request() const { return (request_); }
private:
	char* data_;
	char* key_;
	int len_;
	int off_;
	MC_Request *request_;


};

class MC_Time : public TclObject {
public:
	int command(int argc, const char*const* argv);
private:
	double gettimeofday() const;
};

class MC_Manager : public SRM_AppMgr {
public:
	MC_Manager();
	~MC_Manager();

        /*
	 * Fill in the next message ("data unit") to be sent out. 
	 * Called by the SRM sending code. 
	 */
	virtual int next_ADU(u_char *db, int len, srm_src &id, 
			     int &pkt_type, int &next);
        /*
	 * Called when a new source is identified. We create a new
	 * MCReceiver to handle any data coming from that source. In
	 * this case, the MCReceivers will be forwarding the data
	 * via TCP to the browser so we pass the MCReceiver's 
	 * constructor the TCP Server Handler so it knows how to send. 
	 */
	virtual SRM_PacketHandler *new_source(const srm_src &sid, 
					      int islocal);

	/*
	 * Application fills in the "last sequence number" that it has 
	 * sent out so far. 
	 */
	virtual int periodic_update(u_char * /*buf*/) {
		return 0; 
	}
	
	virtual void handle_request(const srm_src &sid, u_char *pb, 
				    int len);
	virtual void handle_reply(const srm_src &sid, u_char *pb, 
				  int len);
	virtual void handle_pending(const srm_src &sid, u_char *pb, 
				    int len);
	virtual void handle_SA(const srm_src &/*sid*/, u_char * /*pb*/, 
			       int /*len*/) {}
	int command(int argc, const char*const* argv);

	/*
	 * Called when we receive a packet from the net
	 * (from ANY source)
	 */
	virtual void recv(u_char *buf, int len, const srm_src& sid);

private:
	MC_Session *session_;
	/*
	 * hash table routines for actual cache of distributed data
	 */
	void change_key(const char *oldkey, const char *newkey);
	void enter(const char* key, const char* buffer, int len);
	CacheBuf* create(const char* key);
	CacheBuf* remove(const char* key);
	CacheBuf* lookup_by_key(const char* key);
	CacheBuf *setup_page(const char *key, int len);
	void start(const char* key);
	Tcl_HashTable cache_;
	CacheBuf* active_;
	CacheBuf* tail_;
	srm_src sid_;

	friend MC_Reply; // for lookup_by_key()
	friend MC_Request; 
};

#endif // mc_manager_h
