/*
 * Video frame memory representation class definitions.
 *
 * Defined in this file:
 *
 * VidRep - base class for all video representations.
 * Uncompressed - YUV uncompressed representation.
 * Semicompressed - YUV representation at DCT coefficient level.
 * JPEGCompressed - Fully compressed JPEG frame.
 *
 */
#ifndef VIDREP_H
#define VIDREP_H

#include "config.h"
#include "tclcl.h"
#include "module.h"
#include "tclDvmBasic.h"

class VidRep : public TclObject, public Buffer {
public:
  VidRep();
  virtual ~VidRep();
  
  int w_;
  int h_;
  int h_subsample_;
  int v_subsample_;
  int true_w_;
  int true_h_;
  int xoff_;
  int yoff_;
  int resized_;
  u_int32_t ssrc_;
  u_int32_t ts_;
  char *source_id_;

  virtual void copy_geometry(VidRep *temp); 
  virtual int command(int argc, const char*const* argv);
  virtual Buffer *copy() {
    /* XXX This needs to be changed, but we'll do that later. */
    return 0;
  };
};

class Uncompressed : public VidRep {
public:
  Uncompressed();
  virtual ~Uncompressed();

  int self_allocated_;
  
  ByteImage *lum_;
  ByteImage *cr_;
  ByteImage *cb_;

  int lum_id_;
  int cr_id_;
  int cb_id_;

  void dealloc();
  void init(int w, int h, int h_subsample, int v_subsample, int true_w,
	    int true_h, int xoff, int yoff, u_char *ldata, u_char *crdata,
	    u_char *cbdata);
  virtual int command(int argc, const char*const* argv);

  u_char *bp_;
};

class JPEGCompressed : public VidRep {
public:
  JPEGCompressed();
  virtual ~JPEGCompressed();

  int self_allocated_;
  int q_;
  int len_;
  
  u_char *bp_;

  void set(int w, int h, int decimation, int q, int len, 
	   u_int32_t ssrc, u_int32_t ts, u_char *bp);
  void dealloc();
  virtual int command(int argc, const char*const* argv);
};

class Semicompressed : public VidRep {
public:
  Semicompressed();
  virtual ~Semicompressed();

  int self_allocated_;
  
  ScImage *lum_;
  ScImage *cr_;
  ScImage *cb_;

  int lum_id_;
  int cr_id_;
  int cb_id_;

  void allocate();
  void dealloc();
  
  void print();
  void print(int x, int y);

  virtual int command(int argc, const char*const* argv);

  void set(int w, int h, int h_subsample, int v_subsample, 
	   int tw, int th, int xoff, int yoff);
};
#endif
