#
# Copyright (c) 1998 Regents of the University of California.
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
# 	This product includes software developed by the MASH Research
# 	Group at the University of California Berkeley.
# 4. Neither the name of the University nor of the Research Group may be
#    used to endorse or promote products derived from this software without
#    specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Header: /usr/src/mash/repository/mash/mash-1/tcl/mc/ht.tcl,v 1.4 1998/04/21 01:06:49 kwright Exp $

# really just an associative array 
Class HashTable 

HashTable instproc init {} {
}

# return -1 if key doesn't exist already; else return the value
HashTable instproc exists {key} {
	$self instvar table_
	set keys [array names table_]
	set answer [lsearch $keys $key]
	if {$answer == -1} {
		return -1
	} 
	return $table_($key)
}

HashTable instproc change_key {oldkey newkey} {
	$self instvar table_
	if {[$self exists $oldkey] != -1 } {
		set value [$self get $oldkey]
		$self remove $oldkey
		$self create $newkey $value
	}
}

HashTable instproc create {key value} {
	$self instvar table_     
	set table_($key) $value
}

HashTable instproc remove {key} {
	$self instvar table_
	unset table_($key) 
}

HashTable instproc get {key} {
	$self instvar table_
	return $table_($key)
}

HashTable instproc print {} {
	$self instvar table_
	set keys [array names table_]
	puts "HT KEY VALUE:"
	foreach key $keys {
		puts "$key $table_($key)"
	}
}


proc cktest {} {
	HashTable t
	puts "nothing"
	t print
	t create this 0
	puts "this 0"
	t print
	t change_key this that
	puts "that 0"
	t print
}

proc test {} {
	HashTable t
	puts "nothing"
	t print	
	t create this 0
	puts "this 0"
	t print
	t remove this
	puts "nothing"
	t print
	t create this 1
	t create this 2
	puts "this 2"
	t print
	t create that 3
	t create http://www.cs.berkeley.edu _o76
	puts "http://www.cs.berkeley.edu _o76"
	t print
	set data [t get "http://www.cs.berkeley.edu"]
	puts "_o76"
	puts $data
	t remove "http://www.cs.berkeley.edu"
	puts "this 2 that 3"
	t print
	set answer [t exists xyz]
	puts "-1"
	puts $answer
	if {[t exists xyz] == -1} {
		t create xyz 300
	}
	puts "this 2 that 3 xyz 300"
	t print
}

