import AnnounceListenManager URL_Get_Timer URL_Resp_Timer

#
# A control object for the web cache. It takes care of
# communications amoung all caches in a multicast session.
#
Class SRMv2_CacheControl -superclass AnnounceListenManager

#
# The web cache control control. <i>netspec</i> is the multicast
# address passed to the announce listen manager, and <i>wc</i> is
# the web cache component.
#
SRMv2_CacheControl public init { netspec } {
	$self next $netspec

	# default random delay values
	$self set get_c1_ 1.0
	$self set get_c2_ 2.0
	$self set resp_hit_c1_ 1.0
	$self set resp_hit_c2_ 2.0
	$self set resp_miss_c1_ 2.0
	$self set resp_miss_c2_ 3.0
	$self set get_d_ 100.0
	$self set resp_hit_d_ 100.0
	$self set resp_miss_d_ 100.0
}

#
# Create URL_Get_Timer for <i>url</i>.
#
SRMv2_CacheControl public create_get_timer { url } {
	$self instvar gettimer_table_ get_c1_ get_c2_ get_d_ wc_

	# create a URL_Get_Timer
	if ![info exists gettimer_table($url)] {
		puts "wcc: create_get_timer $url"
		set gettimer [new URL_Get_Timer $url $get_c1_ $get_c2_ $get_d_ $self]
		set gettimer_table_($url) $gettimer
	}
}

#
# Called when data arrived from the multicast session thereby
# the get and response timers associated with the url need to
# be canceled.
#
SRMv2_CacheControl public cancel_all_timers { url } {
	$self instvar wc_ gettimer_table_ resptimer_table_

	puts "wcc: cancel all timers $url"

	# cancel all timers and pending messages associated with the url
	if { [info exists gettimer_table_($url)] } {
		$gettimer_table_($url) destroy 
		unset gettimer_table_($url)
	}
	if { [info exists resptimer_table_($url)] } {
		$resptimer_table_($url) destroy  
		unset resptimer_table_($url)
	}
}

#
SRMv2_CacheControl public win_resp_timer { url } {
	$self instvar wc_ gettimer_table_ resptimer_table_

	# cancel the URL_Resp_Timer
	if [info exists resptimer_table_($url)] {
		$resptimer_table_($url) destroy 
		unset resptimer_table_($url)
	}

	# backoff the URL_Get_Timer
	if [info exists gettimer_table_($url)] {
		$gettimer_table_($url) backoff
	}	

	# send a reply pending message
	$self send_announcement WC_RESP_PEND $url		

	# ask the cache to send data associated with the url
	$wc_ send_data $url
}

#
# Send messages when timers expire. If a get message is
# being sent, this method also set a local resp timer. If
# a pending message is being sent, this method also cancel
# the assocaited resp timer.
#
SRMv2_CacheControl public send_announcement { type url { args "" } } {
	$self instvar resptimer_table_ resp_miss_c1_ resp_miss_c2_ \
		resp_miss_d_
	
	switch -- $type {
		WC_URL_GET {
			puts "wcc: announce WC_URL_GET"
			$self announce "WC_URL_GET $url"

			# should not create multiple get timers for the
			# same url. this is ok because we forget about
			# the timer in the array once the srm receiver
			# all the data.
			if ![info exists resptimer_table_($url)] {
				set resptimer_table_($url) \
					[new URL_Resp_Timer $url \
					$resp_miss_c1_ $resp_miss_c2_ \
					$resp_miss_d_ $self]
			}
		}
		WC_RESP_PEND {
			puts "wcc: announce WC_RESP_PEND"
			$self announce "WC_RESP_PEND $url"
		}
	}
}

#
# Receive messages from other caches. There are currently the
# following messages:
# WC_URL_GET means that another cache has won the timer war so
# this cache should backoff its get timer. At the same time,
# it initiates a response timer, which decides which cache will
# send the data back to the session.
# WC_RESP_PEND means that another cache has won the response
# timer war and is currently getting/sending the data. This
# cancels the response timer, and backoffs the get timer.
#
SRMv2_CacheControl public recv_announcement { addr mesg len } {
	$self instvar gettimer_table_ resptimer_table_ wc_ \
		resp_hit_c1_ resp_hit_c2_ resp_miss_c1_ resp_miss_c2_ \
		resp_hit_d_ resp_miss_d_

	puts "wcc: recv $mesg"
	
	set type [lindex $mesg 0]
	set url [lindex $mesg 1]

	switch -- $type {
		WC_URL_GET {
			# backoff its own URL_Get_Timer
			if [info exists gettimer_table_($url)] {
				$gettimer_table_($url) backoff
			}

			# check whether it has data in the local cache
			if { [$wc_ hit $url] != "" } {
				set c1 $resp_hit_c1_
				set c2 $resp_hit_c2_
				set d $resp_hit_d_
			} else {
				set c1 $resp_miss_c1_
				set c2 $resp_miss_c2_
				set d $resp_miss_d_
			}

			# again, do not create multiple timers, because it
			# confuses the destroy timer operation.
			if ![info exists resptimer_table_($url)] {
				set resptimer [new URL_Resp_Timer $url $c1 $c2 $d $self]
				set resptimer_table_($url) $resptimer
			}
		}
		WC_RESP_PEND {
			# cancel its URL_Resp_Timer
			if [info exists resptimer_table_($url)] { 
				$resptimer_table_($url) destroy 
				unset resptimer_table_($url)
			}
			
			# backoff the URL_Get_Timer
			if [info exists gettimer_table_($url)] {
				$gettimer_table_($url) backoff
			}			
		}
	}
}


