/*********************************************************************
 * This Software is copyright INRIA. 1997.
 * 
 * INRIA holds all the ownership rights on the Software. The scientific
 * community is asked to use the SOFTWARE in order to test and evaluate
 * it.
 * 
 * INRIA freely grants the right to use the Software. Any use or
 * reproduction of this Software to obtain profit or for commercial ends
 * being subject to obtaining the prior express authorization of INRIA.
 * 
 * INRIA authorizes any reproduction of this Software
 * 
 *  - in limits defined in clauses 9 and 10 of the Berne agreement for
 * the protection of literary and artistic works respectively specify in
 * their paragraphs 2 and 3 authorizing only the reproduction and quoting
 * of works on the condition that :
 * 
 *  - "this reproduction does not adversely affect the normal
 * exploitation of the work or cause any unjustified prejudice to the
 * legitimate interests of the author".
 * 
 *  - that the quotations given by way of illustration and/or tuition
 * conform to the proper uses and that it mentions the source and name of
 * the author if this name features in the source",
 * 
 *  - under the condition that this file is included with any reproduction.
 * 
 * Any commercial use made without obtaining the prior express agreement
 * of INRIA would therefore constitute a fraudulent imitation.
 * 
 * The Software being currently developed, INRIA is assuming no
 * liability, and should not be responsible, in any manner or any case,
 * for any direct or indirect damages sustained by the user.
 * 
 * Furthermore, INRIA grants for free the right to adapt, modify or integrate
 * the SOFTWARE in another program, provided that any derived work 
 * should be distributed in the same conditions as the SOFTWARE. 
 *********************************************************************
 */
/*
 * RemoteController.java - client applet.
 * Author:  Tie Liao (Tie.Liao@inria.fr).
 * Created: 6 June 1996.
 * Updated: no.
 */

import java.awt.*;
import java.applet.*;
import java.net.*;
import java.util.*;
import java.io.*;

/**
 * Client program running in a Java-enabled browser. It just
 * receives data from multicast deamon and displays in browser's
 * right frame.
 */
public class RemoteController extends Applet implements Runnable {
    private final static String StatusReq = "status:";
    private final static String MsgReq = "msg:";
    private final static String URLReq = "show:";
    private final static String NOPReq = "nop";

    private URL base;
    private String host;
    private int port;
    private Socket sock = null;
    DataInputStream input;

    private Thread thread;

    private String status = null;
    private String msg = null;

    public void init() {
	base = getDocumentBase();
	String browserFrame = getParameter("window");
	host = base.getHost();
	port = base.getPort();

	System.out.println("init host " + host + " port " + port + " frame " + browserFrame);
    }
    public void start() {
	System.out.println("starting...");

	try {
	    sock = new Socket(host, port);
	    input = new DataInputStream(sock.getInputStream());

	    OutputStream output = sock.getOutputStream();
	    int len = getClass().getName().length();
	    byte buff[] = new byte[len];

	    getClass().getName().getBytes(0, len, buff, 0);
	    output.write(buff, 0, len);
	    output.flush();
	} catch (IOException e) {
	    System.out.println("can't create socket " + e);
	    return;
	}

	thread = new Thread(this);
	thread.start();
    }
    public void stop() {
	System.out.println("stopped.");

	if (sock != null) {
	    try {
		sock.close();
	    } catch (Exception e) {
		System.out.println("RemoteController: close() error: " + e);
	    }
	}
	sock = null;
	thread = null;
    }
    private void lost() {
	stop();
	status = "ERROR: applet connection lost";
	repaint();
    }

    public void paint(Graphics g) {
	if (status != null) {
	    getAppletContext().showStatus(status);
	}

	if (msg != null) {
//	    Font f = new Font("Helvetica", 0, 10);
//	    g.setFont(f);

	    int xoff = 2;
	    int yoff = g.getFontMetrics().getHeight()/2 + 2;
	    g.drawString(msg, xoff, yoff);
	}
    }
    private void paintDocument(String doc) {
	try {
	    AppletContext ac = getAppletContext();
	    URL url;

	    if (doc.startsWith("/"))
		url = new URL("http", host, port, doc);
	    else
		url = new URL(doc);

	    getAppletContext().showDocument(url, getParameter("window"));
	} catch (MalformedURLException e) {
	    System.out.println("Invalid URL" + e);
	}
    }
    public void run() {
	String data;

	while (thread != null) {
	    try {
		data = input.readLine();
	    } catch (Exception e) {
		lost();
		return;
	    }

	    System.out.println("req " + data);
	    if (data == null) {
		lost();
		return;
	    }
	    if (data.startsWith(StatusReq)) {
		status = data.substring(StatusReq.length());
		repaint();
	    } else if (data.startsWith(MsgReq)) {
		msg = data.substring(MsgReq.length(), data.length());
		repaint();
	    } else if (data.startsWith(URLReq)) {
		paintDocument(data.substring(URLReq.length(), data.length()));
	    } else if (data.equals(NOPReq)) {
		/* nop */
	    } else {
		System.out.println("Unknown req: " + data);
	    }
	}
    }
}
